/* /////////////////////////////////////////////////////////////////////////////
 * File:        inetstl_filesystem_traits.h (formerly MInetEnm.h)
 *
 * Purpose:     Contains the filesystem_traits template class, and ANSI and
 *              Unicode specialisations thereof.
 *
 * Created:     30th April 1999
 * Updated:     25th April 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/inetstl
 *                          http://www.inetstl.org/
 *
 *              email:      submissions@inetstl.org  for submissions
 *                          admin@inetstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef INETSTL_INCL_H_INETSTL_FILESYSTEM_TRAITS
#define INETSTL_INCL_H_INETSTL_FILESYSTEM_TRAITS

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define _INETSTL_VER_H_INETSTL_FILESYSTEM_TRAITS_MAJOR       1
# define _INETSTL_VER_H_INETSTL_FILESYSTEM_TRAITS_MINOR       2
# define _INETSTL_VER_H_INETSTL_FILESYSTEM_TRAITS_REVISION    1
# define _INETSTL_VER_H_INETSTL_FILESYSTEM_TRAITS_EDIT        17
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef INETSTL_INCL_H_INETSTL
# include "inetstl.h"                        // Include the InetSTL root header
#endif /* !INETSTL_INCL_H_INETSTL */
#ifndef STLSOFT_INCL_H_STLSOFT_SAPCAST
# include "stlsoft_sap_cast.h"               // sap_cast
#endif /* !STLSOFT_INCL_H_STLSOFT_SAPCAST */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _INETSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::inetstl */
namespace inetstl
{
# else
/* Define stlsoft::inetstl_project */

namespace stlsoft
{

namespace inetstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_INETSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup libraries InetSTL Libraries
/// \brief The individual libraries

/// \weakgroup libraries_filesystem File-System Library
/// \ingroup libraries
/// \brief This library provides facilities for defining and manipulating file-system objects

/// \weakgroup inetstl_filesystem_library File-System Library (InetSTL)
/// \ingroup InetSTL libraries_filesystem
/// \brief This library provides facilities for defining and manipulating file-system objects for the WinInet API
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

#ifdef __STLSOFT_DOCUMENTATION_SKIP_SECTION
/// Traits for accessing the correct file-system functions for a given character type
///
/// filesystem_traits is a traits class for determining the correct file-system
/// structures and functions for a given character type.
///
/// \param C The character type
template <ss_typename_param_k C>
struct filesystem_traits
{
public:
    /// The character type
    typedef C               char_type;
    /// The size type
    typedef is_size_t       size_type;
    /// The difference type
    typedef is_ptrdiff_t    difference_type;
    /// The find data type
    typedef WIN32_FIND_DATA find_data_type;     // Placeholder only

public:
    // General string handling

    /// Copies the contents of \c src to \c dest
    static char_type    *str_copy(char_type *dest, char_type const *src);
    /// Appends the contents of \c src to \c dest
    static char_type    *str_cat(char_type *dest, char_type const *src);
    /// Comparies the contents of \c src and \c dest
    static is_int_t     str_compare(char_type const *s1, char_type const *s2);
    /// Comparies the contents of \c src and \c dest in a case-insensitive fashion
    static is_int_t     str_compare_no_case(char_type const *s1, char_type const *s2);
    /// Evaluates the length of \c src
    static size_type    str_len(char_type const *src);

    // File-system entry names

    /// Appends a file-system path separator to \c dir if one does not exist
    static char_type    *ensure_dir_end(char_type *dir);
    /// Returns \c true if dir is \c "." or \c ".."
    static is_bool_t    is_dots(char_type const *dir);
    /// Returns the path separator
    ///
    /// This is the separator that is used to separate multiple paths on the operating system. On UNIX it is ':'
    static char_type    path_separator();
    /// Returns the path name separator
    ///
    /// This is the separator that is used to separate parts of a path on the operating system. On UNIX it is '/'
    static char_type    path_name_separator();
    /// Returns the wildcard pattern that represents all possible matches
    ///
    /// \note On Win32 it is '*.*'
    static char_type const *pattern_all();
    /// Gets the full path name into the given buffer, returning a pointer to the file-part
    static is_dword_t   get_full_path_name(HINTERNET hconn, char_type const *fileName, is_dword_t cchBuffer, char_type *buffer, char_type **ppFile);
    /// Gets the full path name into the given buffer
    static is_dword_t   get_full_path_name(HINTERNET hconn, char_type const *fileName, is_dword_t cchBuffer, char_type *buffer);

    // Internet connectivity

    /// Opens a WinInet session
    static HINTERNET    internet_open(char_type const *agent, is_dword_t accessType, char_type const *proxy, char_type const *proxyBypass, is_dword_t flags);
    /// Makes a connection to a FTP or HTTP site
    static HINTERNET    internet_connect(HINTERNET hsess, char_type const *server, INTERNET_PORT port, char_type const *userName, char_type const *password, is_dword_t service, is_dword_t flags, is_dword_t context);

    // FindFile() API

    /// Initiate a file-system search
    static HINTERNET    find_first_file(HINTERNET hconn, char_type const *spec, find_data_type *findData, is_dword_t flags = 0, is_dword_t context = 0);
    /// Advance a given file-system search
    static is_bool_t    find_next_file(HANDLE h, find_data_type *findData);

    // File system state

    /// Sets the current directory to \c dir
    static is_bool_t    set_current_directory(HINTERNET hconn, char_type const *dir);
    /// Retrieves the name of the current directory into \c buffer up to a maximum of \c cchBuffer characters
    static is_uint_t    get_current_directory(HINTERNET hconn, is_size_t &cchBuffer, char_type *buffer);

    /// Returns whether a file exists or not
    static is_bool_t    file_exists(HINTERNET hconn, char_type const *fileName);

    // File system control

    /// Creates a directory
    static is_bool_t    create_directory(HINTERNET hconn, char_type const *dir);
    /// Deletes a directory
    static is_bool_t    remove_directory(HINTERNET hconn, char_type const *dir);

    /// Delete a file
    static is_bool_t    delete_file(HINTERNET hconn, char_type const *file);
    /// Rename a file
    static is_bool_t    rename_file(HINTERNET hconn, char_type const *currentName, char_type const *newName);

/*  BOOL WINAPI FtpGetFile(
HINTERNET hConnect, 
LPCTSTR lpszRemoteFile, 
LPCTSTR lpszNewFile, 
BOOL fFailIfExists, 
DWORD dwFlagsAndAttributes, 
DWORD dwFlags, 
DWORD dwContext);

    BOOL WINAPI FtpOpenFile(
HINTERNET hConnect, 
LPCTSTR lpszFileName, 
DWORD dwAccess, 
DWORD dwFlags, 
DWORD dwContext);

    BOOL WINAPI FtpPutFile(
HINTERNET hConnect, 
LPCTSTR lpszLocalFile, 
LPCTSTR lpszNewRemoteFile, 
DWORD dwFlags, 
DWORD dwContext);

    BOOL WINAPI InternetReadFile(
IN HINTERNET hFile, 
IN LPVOID lpBuffer, 
IN DWORD dwNumberOfBytesToRead, 
OUT LPDWORD lpdwNumberOfBytesRead);

    BOOL InternetReadFileEx (
HINTERNET hFile,
LPINTERNET_BUFFERS lpBuffersOut, 
DWORD dwFlags,
DWORD dwContext);
 */
};

#else

template <ss_typename_param_k C>
struct filesystem_traits;

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct filesystem_traits<is_char_a_t>
{
public:
    typedef is_char_a_t         char_type;
    typedef is_size_t           size_type;
    typedef is_ptrdiff_t        difference_type;
    typedef WIN32_FIND_DATAA    find_data_type;

public:
    // General string handling
    static char_type *str_copy(char_type *dest, char_type const *src)
    {
        return lstrcpyA(dest, src);
    }

    static char_type *str_cat(char_type *dest, char_type const *src)
    {
        return lstrcatA(dest, src);
    }

    static is_int_t str_compare(char_type const *s1, char_type const *s2)
    {
        return lstrcmpA(s1, s2);
    }

    static is_int_t str_compare_no_case(char_type const *s1, char_type const *s2)
    {
        return lstrcmpiA(s1, s2);
    }

    static size_type str_len(char_type const *src)
    {
        return static_cast<size_type>(lstrlenA(src));
    }

    // File-system entry names
    static char_type *ensure_dir_end(char_type *dir)
    {
        char_type       *end;
        char_type const separator = (NULL == strchr(dir, '/') && NULL != strchr(dir, '\\')) ? '\\' : '/';

        for(end = dir; *end != '\0'; ++end)
        {}

        if( dir < end &&
            *(end - 1) != separator)
        {
            *end        =   separator;
            *(end + 1)  =   '\0';
        }

        return dir;
    }

    static is_bool_t is_dots(char_type const *dir)
    {
        return  dir != 0 &&
                dir[0] == '.' &&
                (   dir[1] == '\0' ||
                    (    dir[1] == '.' &&
                        dir[2] == '\0'));
    }

    static char_type path_separator()
    {
        return ';';
    }

    static char_type path_name_separator()
    {
        return '\\';
    }

    static char_type const *pattern_all()
    {
        return "*.*";
    }

    static is_dword_t get_full_path_name(HINTERNET hconn, char_type const *fileName, is_dword_t cchBuffer, char_type *buffer, char_type **ppFile)
    {
        char_type const separator = (NULL == strchr(fileName, '/') && NULL != strchr(fileName, '\\')) ? '\\' : '/';
        char_type       fullPath[1 + _MAX_PATH];

        if(separator != *fileName)
        {
            is_size_t   cchBuffer   =   stlsoft_num_elements(fullPath);

            get_current_directory(hconn, cchBuffer, fullPath);
            ensure_dir_end(fullPath);
            str_cat(fullPath, fileName);

            fileName = fullPath;
        }

        size_type       len =   str_len(fileName);

        if(cchBuffer < len)
        {
            len = cchBuffer;
        }

        strncpy(buffer, fileName, cchBuffer);

        if(NULL != ppFile)
        {
            char_type const *pRSlash        =   strrchr(buffer, '/');
            char_type const *pRBackSlash    =   strrchr(buffer, '\\');

            if(pRSlash < pRBackSlash)
            {
                pRSlash = pRBackSlash;
            }

            if(NULL == pRSlash)
            {
                *ppFile = NULL;
            }
            else
            {
                *ppFile = const_cast<char_type*>(pRSlash) + 1;
            }
        }

        return len;
    }

    static is_dword_t get_full_path_name(HINTERNET hconn, char_type const *fileName, is_dword_t cchBuffer, char_type *buffer)
    {
        char_type *pFile;

        return get_full_path_name(hconn, fileName, cchBuffer, buffer, &pFile);
    }

    // Internet connectivity
    static HINTERNET internet_open(char_type const *agent, is_dword_t accessType, char_type const *proxy, char_type const *proxyBypass, is_dword_t flags)
    {
        return ::InternetOpenA(agent, accessType, proxy, proxyBypass, flags);
    }

    static HINTERNET internet_connect(HINTERNET hsess, char_type const *server, INTERNET_PORT port, char_type const *userName, char_type const *password, is_dword_t service, is_dword_t flags, is_dword_t context)
    {
        return ::InternetConnectA(hsess, server, port, userName, password, service, flags, context);
    }

    // FindFile() API
    static HINTERNET find_first_file(HINTERNET hconn, char_type const *spec, find_data_type *findData, is_dword_t flags = 0, is_dword_t context = 0)
    {
        return ::FtpFindFirstFileA(hconn, spec, findData, flags, context);
    }

    static is_bool_t find_next_file(HANDLE h, find_data_type *findData)
    {
        return FALSE != ::InternetFindNextFileA(h, findData);
    }

    // File system state
    static is_bool_t set_current_directory(HINTERNET hconn, char_type const *dir)
    {
        return ::FtpSetCurrentDirectoryA(hconn, dir) != FALSE;
    }

    static is_uint_t get_current_directory(HINTERNET hconn, is_size_t &cchBuffer, char_type *buffer)
    {
        return ::FtpGetCurrentDirectoryA(hconn, buffer, sap_cast<unsigned long*>(&cchBuffer));
    }

    static is_bool_t file_exists(HINTERNET hconn, char_type const *fileName)
    {
        find_data_type  data;
        HINTERNET       hFind = find_first_file(hconn, fileName, &data);

        return (NULL == hFind) ? false : (::InternetCloseHandle(hFind), true);
    }

    // File system control
    static is_bool_t create_directory(HINTERNET hconn, char_type const *dir)
    {
        return FALSE != ::FtpCreateDirectoryA(hconn, dir);
    }

    static is_bool_t remove_directory(HINTERNET hconn, char_type const *dir)
    {
        return FALSE != ::FtpRemoveDirectoryA(hconn, dir);
    }

    static is_bool_t delete_file(HINTERNET hconn, char_type const *file)
    {
        return FALSE != ::FtpDeleteFileA(hconn, file);
    }

    static is_bool_t rename_file(HINTERNET hconn, char_type const *currentName, char_type const *newName)
    {
        return FALSE != ::FtpRenameFileA(hconn, currentName, newName);
    }
};

#ifdef __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX
template <>
#endif /* __STLSOFT_CF_TEMPLATE_SPECIALISATION_SYNTAX */
struct filesystem_traits<is_char_w_t>
{
public:
    typedef is_char_w_t         char_type;
    typedef is_size_t           size_type;
    typedef is_ptrdiff_t        difference_type;
    typedef WIN32_FIND_DATAW    find_data_type;

public:
    // General string handling
    static char_type *str_copy(char_type *dest, char_type const *src)
    {
        return lstrcpyW(dest, src);
    }

    static char_type *str_cat(char_type *dest, char_type const *src)
    {
        return lstrcatW(dest, src);
    }

    static is_int_t str_compare(char_type const *s1, char_type const *s2)
    {
        return lstrcmpW(s1, s2);
    }

    static is_int_t str_compare_no_case(char_type const *s1, char_type const *s2)
    {
        return lstrcmpiW(s1, s2);
    }

    static size_type str_len(char_type const *src)
    {
        return static_cast<size_type>(lstrlenW(src));
    }

    // File-system entry names
    static char_type *ensure_dir_end(char_type *dir)
    {
        char_type       *end;
        char_type const separator = (NULL == wcschr(dir, L'/') && NULL != wcschr(dir, L'\\')) ? L'\\' : L'/';

        for(end = dir; *end != L'\0'; ++end)
        {}

        if( dir < end &&
            *(end - 1) != separator)
        {
            *end        =   separator;
            *(end + 1)  =   L'\0';
        }

        return dir;
    }

    static is_bool_t is_dots(char_type const *dir)
    {
        return  dir != 0 &&
                dir[0] == '.' &&
                (   dir[1] == L'\0' ||
                    (    dir[1] == L'.' &&
                        dir[2] == L'\0'));
    }

    static char_type path_separator()
    {
        return L';';
    }

    static char_type path_name_separator()
    {
        return L'\\';
    }

    static char_type const *pattern_all()
    {
        return L"*.*";
    }

    static is_dword_t get_full_path_name(HINTERNET hconn, char_type const *fileName, is_dword_t cchBuffer, char_type *buffer, char_type **ppFile)
    {
        char_type const separator = (NULL == wcschr(fileName, L'/') && NULL != wcschr(fileName, L'\\')) ? L'\\' : L'/';
        char_type       fullPath[1 + _MAX_PATH];

        if(separator != *fileName)
        {
            is_size_t   cchBuffer   =   stlsoft_num_elements(fullPath);

            get_current_directory(hconn, cchBuffer, fullPath);
            ensure_dir_end(fullPath);
            str_cat(fullPath, fileName);

            fileName = fullPath;
        }

        size_type       len =   str_len(fileName);

        if(cchBuffer < len)
        {
            len = cchBuffer;
        }

        wcsncpy(buffer, fileName, cchBuffer);

        if(NULL != ppFile)
        {
            char_type const *pRSlash        =   wcsrchr(buffer, L'/');
            char_type const *pRBackSlash    =   wcsrchr(buffer, L'\\');

            if(pRSlash < pRBackSlash)
            {
                pRSlash = pRBackSlash;
            }

            if(NULL == pRSlash)
            {
                *ppFile = NULL;
            }
            else
            {
                *ppFile = const_cast<char_type*>(pRSlash) + 1;
            }
        }

        return len;
    }

    static is_dword_t get_full_path_name(HINTERNET hconn, char_type const *fileName, is_dword_t cchBuffer, char_type *buffer)
    {
        char_type *pFile;

        return get_full_path_name(hconn, fileName, cchBuffer, buffer, &pFile);
    }

    // Internet connectivity
    static HINTERNET internet_open(char_type const *agent, is_dword_t accessType, char_type const *proxy, char_type const *proxyBypass, is_dword_t flags)
    {
        return ::InternetOpenW(agent, accessType, proxy, proxyBypass, flags);
    }

    static HINTERNET internet_connect(HINTERNET hsess, char_type const *server, INTERNET_PORT port, char_type const *userName, char_type const *password, is_dword_t service, is_dword_t flags, is_dword_t context)
    {
        return ::InternetConnectW(hsess, server, port, userName, password, service, flags, context);
    }

    // FindFile() API
    static HINTERNET find_first_file(HINTERNET hconn, char_type const *spec, find_data_type *findData, is_dword_t flags = 0, is_dword_t context = 0)
    {
        return ::FtpFindFirstFileW(hconn, spec, findData, flags, context);
    }

    static is_bool_t find_next_file(HANDLE h, find_data_type *findData)
    {
        return FALSE != ::InternetFindNextFileW(h, findData);
    }

    // File system state
    static is_bool_t set_current_directory(HINTERNET hconn, char_type const *dir)
    {
        return ::FtpSetCurrentDirectoryW(hconn, dir) != FALSE;
    }

    static is_uint_t get_current_directory(HINTERNET hconn, is_size_t &cchBuffer, char_type *buffer)
    {
        return ::FtpGetCurrentDirectoryW(hconn, buffer, sap_cast<unsigned long*>(&cchBuffer));
    }

    /// Returns whether a file exists or not
    static is_bool_t file_exists(HINTERNET hconn, char_type const *fileName)
    {
        find_data_type  data;
        HINTERNET       hFind = find_first_file(hconn, fileName, &data);

        return (NULL == hFind) ? false : (::InternetCloseHandle(hFind), true);
    }

    // File system control
    static is_bool_t create_directory(HINTERNET hconn, char_type const *dir)
    {
        return FALSE != ::FtpCreateDirectoryW(hconn, dir);
    }

    static is_bool_t remove_directory(HINTERNET hconn, char_type const *dir)
    {
        return FALSE != ::FtpRemoveDirectoryW(hconn, dir);
    }

    static is_bool_t delete_file(HINTERNET hconn, char_type const *file)
    {
        return FALSE != ::FtpDeleteFileW(hconn, file);
    }

    static is_bool_t rename_file(HINTERNET hconn, char_type const *currentName, char_type const *newName)
    {
        return FALSE != ::FtpRenameFileW(hconn, currentName, newName);
    }
};

#endif /* __STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group inetstl_filesystem_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _INETSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
} // namespace inetstl
# else
} // namespace inetstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_INETSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* INETSTL_INCL_H_INETSTL_FILESYSTEM_TRAITS */

/* ////////////////////////////////////////////////////////////////////////// */
