/* /////////////////////////////////////////////////////////////////////////////
 * File:        winstl_error_desc.h
 *
 * Purpose:     Converts a Win32 error code to a printable string.
 *
 * Created:     13th July 2003
 * Updated:     25th April 2004
 *
 * Author:      Matthew Wilson, Synesis Software Pty Ltd.
 *
 * License:     (Licensed under the Synesis Software Standard Source License)
 *
 *              Copyright (C) 2002-2003, Synesis Software Pty Ltd.
 *
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/winstl
 *                          http://www.winstl.org/
 *
 *              email:      submissions@winstl.org  for submissions
 *                          admin@winstl.org        for other enquiries
 *
 *              Redistribution and use in source and binary forms, with or
 *              without modification, are permitted provided that the following
 *              conditions are met:
 *
 *              (i) Redistributions of source code must retain the above
 *              copyright notice and contact information, this list of
 *              conditions and the following disclaimer.
 *
 *              (ii) Any derived versions of this software (howsoever modified)
 *              remain the sole property of Synesis Software.
 *
 *              (iii) Any derived versions of this software (howsoever modified)
 *              remain subject to all these conditions.
 *
 *              (iv) Neither the name of Synesis Software nor the names of any
 *              subdivisions, employees or agents of Synesis Software, nor the
 *              names of any other contributors to this software may be used to
 *              endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              This source code is provided by Synesis Software "as is" and any
 *              warranties, whether expressed or implied, including, but not
 *              limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software be liable for any direct, indirect,
 *              incidental, special, exemplary, or consequential damages
 *              (including, but not limited to, procurement of substitute goods
 *              or services; loss of use, data, or profits; or business
 *              interruption) however caused and on any theory of liability,
 *              whether in contract, strict liability, or tort (including
 *              negligence or otherwise) arising in any way out of the use of
 *              this software, even if advised of the possibility of such
 *              damage.
 *
 * ////////////////////////////////////////////////////////////////////////// */


#ifndef _INCL_WINSTL_H_WINSTL_ERROR_DESC
#define _INCL_WINSTL_H_WINSTL_ERROR_DESC

#ifndef __STLSOFT_DOCUMENTATION_SKIP_SECTION
# define _WINSTL_VER_H_WINSTL_ERROR_DESC_MAJOR       1
# define _WINSTL_VER_H_WINSTL_ERROR_DESC_MINOR       5
# define _WINSTL_VER_H_WINSTL_ERROR_DESC_REVISION    1
# define _WINSTL_VER_H_WINSTL_ERROR_DESC_EDIT        15
#endif /* !__STLSOFT_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#ifndef WINSTL_INCL_H_WINSTL
# include "winstl.h"                        // Include the WinSTL root header
#endif /* !WINSTL_INCL_H_WINSTL */
#ifndef WINSTL_INCL_H_WINSTL_FUNCTIONS
# include "winstl_functions.h"              // winstl::FormatMessage()
#endif /* !WINSTL_INCL_H_WINSTL_FUNCTIONS */
#ifndef WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS
# include "winstl_filesystem_traits.h"      // load_library
#endif /* !WINSTL_INCL_H_WINSTL_FILESYSTEM_TRAITS */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#ifndef _WINSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
/* There is no stlsoft namespace, so must define ::winstl */
namespace winstl
{
# else
/* Define stlsoft::winstl_project */

namespace stlsoft
{

namespace winstl_project
{

# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

/// \weakgroup winstl_error_library Error Library
/// @{

/* /////////////////////////////////////////////////////////////////////////////
 * Classes
 */

/// String form of a Win32 error
///
/// This class
template<   ss_typename_param_k C
        ,   ss_typename_param_k T = filesystem_traits<C>
        >
class basic_error_desc
{
public:
    /// The character type
    typedef C                       char_type;
    /// The traits_type
    typedef T                       traits_type;
    /// The current parameterisation of the type
    typedef basic_error_desc<C, T>  class_type;

// Construction
public:
    /// Constructor
    ///
    /// \note This uses the calling thread's error current value, obtained by 
    /// calling GetLastError()
    ss_explicit_k basic_error_desc(ws_dword_t error = GetLastError(), char_type const *paths = NULL);
public:
    /// Destructor
    ~basic_error_desc() winstl_throw_0();

// Accessors
public:
    /// The error code
    operator char_type const *() const;

// Members
private:
    char_type   *m_message;

// Not to be implemented
public:
    basic_error_desc(class_type const &);
    basic_error_desc &operator =(class_type const &);
};

/* Typedefs to commonly encountered types. */
/// Instantiation of the basic_error_desc template for the ANSI character type \c char
typedef basic_error_desc<ws_char_a_t>   error_desc_a;
/// Instantiation of the basic_error_desc template for the Unicode character type \c wchar_t
typedef basic_error_desc<ws_char_w_t>   error_desc_w;
/// Instantiation of the basic_error_desc template for the Win32 character type \c TCHAR
typedef basic_error_desc<TCHAR>         error_desc;

/* ////////////////////////////////////////////////////////////////////////// */

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline basic_error_desc<C, T>::basic_error_desc(ws_dword_t error /* = ::GetLastError() */, char_type const *paths /* = NULL */)
    : m_message(NULL)
{
    ws_dword_t  cch =   0;

    if(NULL != paths)
    {
        HINSTANCE   hinstSource =   traits_type::load_library(paths);

        if(NULL != hinstSource)
        {
            cch =   FormatMessage(error, hinstSource, &m_message);

            traits_type::free_library(hinstSource);
        }
    }

    if(NULL == m_message)
    {
        cch = FormatMessage(error, NULL, &m_message);
    }

    if(0 == cch)
    {
        m_message = NULL;
    }
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline basic_error_desc<C, T>::~basic_error_desc() winstl_throw_0()
{
    if(m_message != NULL)
    {
        ::LocalFree(m_message);
    }
}

template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline basic_error_desc<C, T>::operator ss_typename_type_k basic_error_desc<C, T>::char_type const *() const
{
    return (NULL != m_message) ? m_message : "";
}

/* /////////////////////////////////////////////////////////////////////////////
 * String access shims
 */

#ifndef __STLSOFT_CF_TEMPLATE_SHIMS_NOT_SUPPORTED

/// \brief Returns the corresponding C-string pointer of \c e
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline C const *c_str_ptr_null(basic_error_desc<C, T> const &e)
{
    C const *p  =   e;

    return ('\0' != *e) ? p : NULL;
}

/// \brief Returns the corresponding C-string pointer of \c e
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline C const *c_str_ptr(basic_error_desc<C, T> const &e)
{
    return e;
}

/// \brief Returns the corresponding C-string pointer of \c e
template<   ss_typename_param_k C
        ,   ss_typename_param_k T
        >
inline C const *get_ptr(basic_error_desc<C, T> const &e)
{
    return e;
}

#endif /* !__STLSOFT_CF_TEMPLATE_SHIMS_NOT_SUPPORTED */

////////////////////////////////////////////////////////////////////////////////
// Unit-testing

#ifdef STLSOFT_UNITTEST

namespace unittest
{
    ss_bool_t test_winstl_error_desc(unittest_reporter *r)
    {
        ss_bool_t   bSuccess    =   true;

        r->set_project("WinSTL");
        r->set_file(__FILE__);
        r->set_component("error_desc");

#if 0
        if(<<TODO>>)
        {
            r->report("<<TODO>> failed ", __LINE__);
            bSuccess = false;
        }
#endif /* 0 */

        return bSuccess;
    }

    unittest_initialiser    unittest_winstl_error_desc(test_winstl_error_desc);

} // namespace unittest

#endif /* STLSOFT_UNITTEST */

/* ////////////////////////////////////////////////////////////////////////// */

/// @} // end of group winstl_error_library

/* ////////////////////////////////////////////////////////////////////////// */

#ifndef _WINSTL_NO_NAMESPACE
# ifdef _STLSOFT_NO_NAMESPACE
} // namespace winstl
# else
} // namespace winstl_project
} // namespace stlsoft
# endif /* _STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 *
 * The string access shims exist either in the stlsoft namespace, or in the
 * global namespace. This is required by the lookup rules.
 *
 */

#ifndef _WINSTL_NO_NAMESPACE
# ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
# else /* ? _STLSOFT_NO_NAMESPACE */
/* There is no stlsoft namespace, so must define in the global namespace */
# endif /* !_STLSOFT_NO_NAMESPACE */

using ::winstl::c_str_ptr_null;

using ::winstl::c_str_ptr;

using ::winstl::get_ptr;

# ifndef _STLSOFT_NO_NAMESPACE
} // namespace stlsoft
# else /* ? _STLSOFT_NO_NAMESPACE */
/* There is no stlsoft namespace, so must define in the global namespace */
# endif /* !_STLSOFT_NO_NAMESPACE */
#endif /* !_WINSTL_NO_NAMESPACE */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* _INCL_WINSTL_H_WINSTL_ERROR_DESC */

/* ////////////////////////////////////////////////////////////////////////// */
